#!/usr/bin/perl
# -*- perl -*-
#
# Produce a ROM binary from a number of files
#

use bytes;

sub num($) {
    my($n) = @_;

    return ($n =~ /^0/) ? oct $n : $n+0;
}

$file = shift(@ARGV);
$size = shift(@ARGV);

$data = "\xff" x $size;		# Fill unused memory with FF

foreach $input (@ARGV) {
    if ($input !~ /^(.*)\=([0-9a-fx]+)(|,([0-9a-fx]+))$/i) {
	die "$0: invalid input spec: $input\n";
    }
    $ifile = $1;
    $start = num($2);
    $len   = $3 eq '' ? num($4) : 0;
    
    if ($start >= $size) {
	print STDERR "$0: $input: start beyond output size, ignoring\n";
	next;
    }

    open(IFILE, '<', $ifile) or die "$0: cannot open $ifile: $!\n";
    $ll = read(IFILE, $idata, $len ? $len : $size);
    close(IFILE);

    if ($len && $ll != $len) {
	die "$0: $ifile: could not read $len bytes\n";
    }

    $len = $ll;
    if ($start + $len > $size) {
	$len = $size-$start;
    }

    substr($data, $start, $len, $idata);
}

open(OFILE, '>', $file) or die "$0: cannot open $ofile: $!\n";
print OFILE $data;
close(OFILE);

